use Moops;

# PODNAME: MarpaX::Languages::M4::Role::Impl

# ABSTRACT: M4 implementation role

role MarpaX::Languages::M4::Role::Impl {

    our $VERSION = '0.003'; # VERSION

    use MarpaX::Languages::M4::Role::Builtin;
    use MarpaX::Languages::M4::Role::Logger;
    use MarpaX::Languages::M4::Role::Parser;

    requires 'impl_quote';
    requires 'impl_unquote';
    requires 'impl_appendValue';
    requires 'impl_value';
    requires 'impl_valueRef';
    requires 'impl_parseIncremental';
    requires 'impl_parse';
    requires 'impl_unparsed';
    requires 'impl_setEoi';
    requires 'impl_line';
    requires 'impl_debugfile';
    requires 'impl_rc';

    with 'MarpaX::Languages::M4::Role::Builtin';
    with 'MarpaX::Languages::M4::Role::Logger';
    with 'MarpaX::Languages::M4::Role::Parser';
}

__END__

=pod

=encoding UTF-8

=head1 NAME

MarpaX::Languages::M4::Role::Impl - M4 implementation role

=head1 VERSION

version 0.003

=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan

=head1 SUPPORT

=head2 Bugs / Feature Requests

Please report any bugs or feature requests through the issue tracker
at L<https://rt.cpan.org/Public/Dist/Display.html?Name=MarpaX-Languages-M4>.
You will be notified automatically of any progress on your issue.

=head2 Source Code

This is open source software.  The code repository is available for
public review and contribution under the terms of the license.

L<https://github.com/jddurand/marpax-languages-m4>

  git clone git://github.com/jddurand/marpax-languages-m4.git

=head1 AUTHOR

Jean-Damien Durand <jeandamiendurand@free.fr>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2015 by Jean-Damien Durand.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
